/*globals define, window, CKEDITOR, top, jQuery*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/editor-unity-test',
    'jquery',
    'ep/ui/editor-unity',

    '$ready!'
], function (QUnit, fixture, $, ep) {
    // var fixture = fixture;

    QUnit.module("ep/ui/editor-unity", {
        // setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            '_renderButton',
            '_renderToolbarDropDown',
            '_renderToolbarPlaceholder',
            '_buildToolbar',
            '_buildTextarea',
            '_updateHTML',
            '_buildPlaceholderButtons',
            'checkChars',
            'destroy'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.uiEditorUnity.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });


    QUnit.asyncTest("widget core tests", function () {
        // initialize fixture
        fixture.init();

        var editor = ep(document.getElementById('uieditor_ScribeEditor_01')),
            string = "hello world",
            maxLength = 10,
            availableMemoryElement = $('#availableMemory'),
            availableMemoryValue = (('<p>'+string+'</p>').length / maxLength * 100).toFixed(1),
            commands = {
                dropdown: [
                    {name: 'p'},
                    {name: 'h1'},
                    {name: 'h2'},
                    {name: 'h3'}
                ],
                buttons: [
                    {name: 'bold'},
                    {name: 'epitalic'},
                    {name: 'underline'},
                    {name: 'blockquote'},
                    {name: 'linkTooltip'},
                    {name: 'unlink'},
                    {name: 'insertOrderedList'},
                    {name: 'insertUnorderedList'},
                    {name: 'justifyLeft'},
                    {name: 'justifyCenter'},
                    {name: 'justifyRight'}
                ]
            },
            widget,
            scribe,
            textarea,
            toolbar,
            items,
            i;

        // call widget on editor node
        editor.uiEditorUnity({
            name: 'ScribeEditor_01',
            availableMemory : {
                elem : availableMemoryElement,
                maxLength : maxLength,
                isLengthCheckActive : true
            }
        });


        window.setTimeout(function () {
            editor = ep(document.getElementById('uieditor_ScribeEditor_01'));
            widget = editor.data('uiUiEditorUnity');
            scribe = widget.scribe;

            // check existence of textarea
            textarea = editor.parent().find('textarea.scribe-textarea');
            // test content of corresponding textarea
            QUnit.equal(
                textarea.length,
                1,
                'corresponding textarea exists'
            );

            // set content to scribe
            scribe.setHTML(string);
            // update textarea
            widget._updateHTML();
            window.setTimeout(function() {
                // test content of corresponding textarea
                QUnit.equal(
                    new RegExp(string).test(textarea.val()),
                    true,
                    'corresponding textarea contains the inserted string "' + string + '"'
                );

                // check existence of textarea
                toolbar = editor.parent().find('.toolbar');
                // test content of corresponding toolbar
                QUnit.equal(
                    toolbar.length,
                    1,
                    'toolbar exists'
                );

                // loop over dropdown items
                for (i=0; i<commands.buttons.length; i++) {
                    var item = toolbar.children('button[data-command-name=' + commands.buttons[i].name + ']');
                    // test existence of buttons item
                    QUnit.equal(
                        item.length,
                        1,
                        'entry "' + commands.buttons[i].name + '" exists in toolbar'
                    );
                }

                // get items in toolbar
                items = toolbar.find('.dali-plugin-text-toolbar-dropdown ul li');
                // loop over dropdown items
                for (i=0; i<commands.dropdown.length; i++) {
                    var item = items.eq(i).children('button[data-command-name=' + commands.dropdown[i].name + ']');
                    // test existence of dropdown item
                    QUnit.equal(
                        item.length,
                        1,
                        'entry "' + commands.dropdown[i].name + '" exists in dropdown'
                    );
                }

                QUnit.ok(
                    (new RegExp(availableMemoryValue)).test(parseFloat(availableMemoryElement.text()).toFixed(1)),
                    'availableMemory element contains expected value'
                );

                QUnit.start();
            }, 400);
        }, 500);

    });

    QUnit.asyncTest("placeholder plugin tests", function () {
        // initialize fixture
        fixture.init();

        var editor = ep(document.getElementById('uieditor_ScribeEditor_01')),
            placeholder = [
                {"description": "Bestellnummer", "code": "#Alias"},
                {"description": "Bestelldatum", "code": "#CreationDate[date]"}
            ],
            scribe,
            toolbar,
            btns,
            content,
            i;

        // call widget on editor node
        editor.uiEditorUnity({
            name: 'ScribeEditor_01',
            placeholder: placeholder
        });

        window.setTimeout(function () {
            toolbar = $('.toolbar');
            btns = toolbar.find('.dali-plugin-placeholder-dropdown').find('ul>li');
            editor = ep(document.getElementById('uieditor_ScribeEditor_01'));
            scribe = editor.data('uiUiEditorUnity').scribe;

            QUnit.equal(
                btns.length,
                2,
                'placeholder dropdown has 2 entries'
            );

            // loop over placeholder array
            for (i=0; i<placeholder.length; i++) {
                // test button entry
                QUnit.equal(
                    btns.eq(i).children('button').html(),
                    placeholder[i].description,
                    i + '. placeholder is called "' + placeholder[i].description + '"'
                );
                // test data attribute
                QUnit.equal(
                    btns.eq(i).children('button').data('placeholderValue'),
                    placeholder[i].code,
                    i + '. placeholder is called "' + placeholder[i].code + '"'
                );

                // clear scribe content first
                scribe.setHTML("");

                // set focus on scribe editor
                editor.trigger('focus');
                // execute current placeholder
                scribe.getCommand(placeholder[i].code).execute();
                // get content from scribe editor
                content = scribe.getHTML();
                // test placeholder
                QUnit.equal(
                    new RegExp(placeholder[i].code.replace(/\[|\]/g, function (needle) {return "\\" + needle;})).test(content),
                    true,
                    'placeholder "' + placeholder[i].code + '" was inserted'
                );
            }

            // var selection = new scribe.api.Selection()
            //     // ,range = selection.selection.getRangeAt(0);

            // ep(document.getElementById('uieditor_ScribeEditor_01')).trigger('focus');
            // ep(document.getElementById('uieditor_ScribeEditor_01')).trigger('click');
            // btns.eq(0).trigger('click');

            QUnit.start();
        }, 400);

    });

});